import path from 'path';
import loaderUtils from 'loader-utils';

function rel(p) {
  return path.relative(process.cwd(), p);
}

function processResult(loaderContext, result) {
  if (!result || typeof result !== 'object' || 'code' in result === false) {
    loaderContext.callback(new Error(`The returned result of module ${rel(loaderContext.resource)} is not an object with a 'code' property.`));

    return;
  }

  if (typeof result.code !== 'string' && result.code instanceof Buffer === false) {
    loaderContext.callback(new Error(`The returned code of module ${rel(loaderContext.resource)} is neither a string nor an instance of Buffer.`));

    return;
  }

  (result.dependencies || []).forEach(function (dep) {
    return loaderContext.addDependency(dep);
  });
  // Defaults to false which is a good default here because we assume that
  // results tend to be not cacheable when this loader is necessary
  loaderContext.cacheable(Boolean(result.cacheable));

  loaderContext.callback(null, result.code, result.sourceMap || null, result.ast || null);
}

function valLoader(content) {
  var _this = this;

  var options = loaderUtils.getOptions(this);
  var exports = this.exec(content, this.resource);
  var func = exports && exports.default ? exports.default : exports;

  if (typeof func !== 'function') {
    throw new Error(`Module ${rel(this.resource)} does not export a function as default.`);
  }

  var result = func(options);

  if (result && typeof result.then === 'function') {
    var callback = this.async();

    result.then(function (res) {
      return processResult(_this, res);
    }, callback);

    return;
  }

  // No return necessary because processResult calls this.callback()
  processResult(this, result);
}

export default valLoader;
//# sourceMappingURL=index.js.map